// Geoffrey Zweig (gzweig@microsoft.com)
// Copyright (c) 2009 Microsoft Corporation. All rights reserved.

#include <iostream>
#include <algorithm>
#include <assert.h>
#include "edit_distance.h"

#ifndef _GCC_
# pragma warning( disable:4996 )
#endif

int min_ed(vector<int> &ref, vector<int> &hyp, vector<alignop> &alignment)
{
    // compute the minimum cost edit of the ref into the hyp
    // c(i,j): cost of aligning reference up to and including position i to
    // the hypotheses up to and including position j

    if (ref[0]!=-1 || hyp[0]!=-1) {
        cerr << "Must make 1st position of ref & hyp -1 as a marker\n";
        exit(1);
    }

    const int rlen = (int) ref.size();
    const int hlen = (int) hyp.size();

    int **c = new int *[rlen];
    operation **b = new operation *[rlen];
    int *cstore = new int[rlen*hlen];
    operation *bstore = new operation[rlen*hlen];
    for (int i=0; i<rlen; i++) {
        c[i] = cstore + i*hlen;
        b[i] = bstore + i*hlen; 
    }

    // initialize 
    c[0][0] = 0;
    b[0][0] = null;
    for (int i=1; i<rlen; i++) {
        c[i][0] = c[i-1][0] + 1;
        b[i][0] = del;
    }

    for (int i=1; i<hlen; i++) {
        c[0][i] = c[0][i-1] + 1;
        b[0][i] = ins;
    }

    // do dp
    int dcost, icost;
    for (int i=1; i<rlen; i++) 
    {
        for (int j=1; j<hlen; j++) {
			int bestc = c[i-1][j-1] + ((ref[i]==hyp[j])?(0):(1));
            operation bp = sub;
            if ((dcost=c[i-1][j] + 1) < bestc) {
                bestc = dcost;
                bp = del;
            }
            if ((icost=c[i][j-1] + 1) < bestc) {
                bestc = icost;
                bp = ins;
            }
            c[i][j] = bestc;
            b[i][j] = bp;
        }
    }

    int rpos=rlen-1, hpos=hlen-1;
    int cost = c[rpos][hpos];
    operation op = b[rpos][hpos];
    while (op != null) {
        alignment.push_back(alignop(rpos, hpos, op));
        if (op==sub) {
            rpos--;
            hpos--;
        }
        else if (op==ins) {
            hpos--;
        }
        else if (op==del) {
            rpos--;
        }
        op = b[rpos][hpos];
    }    
    reverse(alignment.begin(), alignment.end());
    
    delete [] c;
    delete [] b;
    delete [] cstore;
    delete [] bstore;

    return cost;
}
