// Geoffrey Zweig (gzweig@microsoft.com)
// Copyright (c) 2009 Microsoft Corporation. All rights reserved.

#ifndef MATCH_FUNCS_H
#define MATCH_FUNCS_H

#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include "model.h"
#include "lm.h"

#ifndef _GCC_
// get rid of deprecated warnings (fopen, sscanf, strcpy)
# pragma warning( disable:4996 )
#endif /* _GCC_ */

using namespace std;

struct doc
{
	map<int, int> count4unit;
	int nwords;
    int label;  // the document's label
	doc(int s) {label = s; nwords=0;}
};

struct idx_entry 
{
	vector<pair<int, float> > tfidfvec;
	int label;
	idx_entry(int s) {label = s;}
	idx_entry() {label=INT_MAX;}
};

struct statetime_cost {  // same as scarftypes::statetime, with an additional float
	lmidx state;
	int t, w; // w is necessary to keep track of value of words that are unk to the lm
	int latpos; // position in lattice if lattice constraints are used
	float score;
	statetime_cost(lmidx s, int time, int word, int lp, float sc) {state=s; t=time; w=word; 
						latpos=lp; score=sc;}
	statetime_cost() {state=0;t=-1;w=INT_MAX;latpos=time_lat_rec::lstart;score=-1e30f;}
};

struct fastMatch 
{
	vector<doc> docs;
	map<int, string> unit4idx, word4idx;
	map<string, int> idx4unit, idx4word;
	int nglen;
	map<vector<int>, int> idx4ngram;
	map<int, vector<int> > ngram4idx;
	vector<float> idf;
	vector<idx_entry> index;
	int ngnum;
	vector<vector<int> > docs4idx;
	vector<int> noccs4doc;  // how many realizations of the document (word) are there?
	float minfrac;  // at least this fraction of a word's realizations must have an index
	                // for the word to be put on the index's reverse lookup
	float phonedelta;  // block size must be within this many units of the expected size
	float fm_lmwt, lengthwt, dm_lmwt;
	size_t maxalphas;
	map<int, int> sumphones4;
	vector<float> length4word;
	vector<float> scratch;
	vector<float> beams;

	// data structures for full decoding
	vector<map<lmrec,float> > alphas, betas;
	map<statetime, vector<statetime_cost> > pred;
	language_model *LM;

	void parse_phones(char *input, vector<int> &output, int &inum);
	vector<set<vector<int> > > xm4;
	bool check_xm;

	void create_index(const char *fname, int ngl, float mf);
	void make_doc(doc &currdoc, vector<int> &phones, int count=1);
	void compute_idf();
	void compute_tfidf_vec(doc &currdoc, vector<pair<int, float> > &tfidfvec);
	float cosine(vector<pair<int, float> > &v1, vector<pair<int, float> > &v2);
	void match(vector<int> &units, vector<pair<int, float> > &matches, int maxn);
	void display_idf();
	void show_units(vector<int> &units);
	void show_tfidf(vector<pair<int, float> > &vec);
	inline float tfidfscore(int w, vector<pair<int, float> > &tfidfvec, vector<int> &units);
	float get_tfidfscore(int w, vector<int> &units);

	void decode(vector<int> &units, int maxl, int maxn);
	pair<lmidx , float> succ(lmidx state, int word); // returns next state and logprob
	vector<pair<int, int> > backtrace(vector<int> &times, FILE *fout); // returns word, time

	void write_index(const char *fname);
	void read_index(const char *fname);
};

#endif


