#!/usr/bin/perl -w

# skips silence in the lattice
# fix bug in silence handling
$maxsil_depth = 3; # allow up to this many ~SIL in a row

die "usage: intersect lattices ref num\n" unless ($#ARGV==2);

$hypf = $ARGV[0];
$reff = $ARGV[1];
$limit = $ARGV[2];

if ($hypf =~ /\.gz\b/o) {
    open(FIN, "gzip -cd $hypf |") or die "unable to open $hypf\n";
} else {	
    open(FIN, $hypf) or die "unable to open $hypf\n";
}

if ($reff =~ /\.gz\b/o) {
    open(RIN, "gzip -cd $reff |") or die "unable to open $reff\n";
} else {	
    open(RIN, $reff) or die "unable to open $reff\n";
}

@ref = ();
%times = ();
%arrived = ();
%exit4 = ();
%variants4 = ();  # word with following silence

# $want = 0;
# $processed = 0;

sub read_ref() {  # in Btr format
    $r = <RIN>;  # the guid
# print "processed $processed\n" if ((++$processed % 1000) == 0);
# exit if ($want==1);
# $want = 1;
# $want = 0 unless ($r =~ /19960510_NPR_ATC#Alan_Liotta\@0001/o);
    $r =~ /^(\S+)\.(tr|pd)/o;
    die "confused by $r\n" unless (defined($1));
    $guid = $1;
    $r = <RIN>; # the transcript  
    $r =~ s/[\r\n]//og;
    $r =~ s/~SIL//og;  # ignore ~SIL in match
    @ref = split(/\s+/o, $r);
    @ref = ("NULL", @ref);
    $r = <RIN>;  # the period
} 

sub read_fm() { # in Bdc format
    %times = ();
    %exit4 = ();
    %variants4 = ();
    my %endsat = ();
    $maxtime = -1;
    $f = <FIN>;  # the guid
    while (($f = <FIN>) !~ /^\./o) {
        $f =~ s/[\r\n]//og;
        @fields = split(/\s+/o, $f);
        die "confused in read_fm\n$f\n" unless ($#fields==2);
        my $st = $fields[0];
        my $et = $fields[1];
        my $word = $fields[2];
        $times{$st} = 1;
        $times{$et} = 1;
        $exit4{"${st}_$word"} .= "$et ";
        $maxtime = ($et>$maxtime)?($et):($maxtime);

        # extend silence chains
        if ($word eq "~SIL") {
            if (defined($endsat{$st - 1})) {
                $to_extend = $endsat{$st - 1};
                $to_extend =~ s/\s+$//o;
                my @words = split(/ /, $to_extend);
                foreach $w (@words) {
                    my @sf = split(/~S/, $w);
                    next if ($#sf > $maxsil_depth);
                    my $nw = "$w" . "~SIL_${st}_${et}";
                    $endsat{$et} .= "$nw ";
                    my $nws = $nw;
                    $nw =~ s/(~SIL_\d+_\d+)+$//o;
                    my ($w, $s, $e) = $nw =~ /(\S+)_(\d+)_(\d+)/o;
                    $exit4{"${s}_$w"} .= "$et ";
                    $variants4{"${w}_${s}_${et}"} .= "$nws ";
# print "${w}_${s}_${et} => $nws\n" if ($want);
                }
            }
        } else {
            $endsat{$et} .= "${word}_${st}_${et} ";
            $variants4{"${word}_${st}_${et}"} .= "${word}_${st}_${et} ";
# print "${word}_${st}_${et} => ${word}_${st}_${et}\n" if ($want);
        }
    }
}

sub link_cmp {
    $a =~ /(\d+) (\d+)/o;
    my $ast = $1;
    my $aet = $2;
    $b =~ /(\d+) (\d+)/o;
    my $bst = $1;
    my $bet = $2;
    if ($ast < $bst) {
        return 1;
    } elsif ($ast > $bst) {
        return -1;
    } elsif ($aet < $bet) {
        return 1;
    } elsif ($aet > $bet) {
        return -1;
    }
    return 0;
}

for ($c=0; $c<$limit; $c++) {
    read_ref();
    read_fm();
# next unless ($want == 1);
    
    %arrived = ();
    $arrived{"0_1"} = 1;
    %preds4 = ();
    %num4 = ();  
    $snum = 0;
    $num4{"0_1"} = $snum++;

    for ($n=1;$n<=$#ref; $n++) {
        $pn = $n-1;
        @newq = ();
        foreach $t (keys(%times)) {
            if (defined($arrived{"${pn}_$t"})) {
                $state = "${t}_$ref[$n]";
                if (defined($exit4{$state})) {
                    @dest = split(/ /, $exit4{$state});
                    foreach $nt (@dest) {
                        next if ($ref[$n] eq "</s>" and $nt != $maxtime); # </s> not at the end screws things up
                        $st = ($nt==$maxtime)?($maxtime):($nt+1);  # will not be continuing from the end
                        $arrived{"${n}_$st"} = 1;
#print "$ref[$n] $st\n" if ($want);
                        $preds4{"${n}_$st"} .= "${pn}_$t "; 
                        $num4{"${n}_$st"} = $snum++ if (not defined $num4{"${n}_$st"});
                        push(@newq, "${n}_$st");
                    }
                }
            }
        }
    }

    # backtrace
    $mw = $#ref;
    if (defined($arrived{"${mw}_$maxtime"})) {
        print "$guid.nc\n";
        @states = ();
        %added = ();
        push(@states, "${mw}_$maxtime");
        $added{"${mw}_$maxtime"} = 1;
        @output = ();
        %processed = ();
        while (defined($s = pop(@states))) {
            $s =~ s/\s+$//o;
            $s =~ /(\d+)_(\d+)/o;
            $refpos = $1;
            $word = $ref[$refpos];
            $end_time = $2;
            $to = $num4{$s};
# print "$s => $num4{$s}\n";
            next unless (defined($plist = $preds4{$s}));
            @preds = split(/\s+/, $plist);
            foreach $p (@preds) {
                push(@states,$p) unless (defined($added{$p}));
                $added{$p} = 1;
                $p =~ /(\d+)_(\d+)/o;
                $start_time = $2;
                $from= $num4{$p};
                $et = ($end_time==$maxtime)?($end_time):($end_time-1);
die "variants for ${word}_${start_time}_${et} not defined" unless (defined( $variants4{"${word}_${start_time}_${et}"}));
                $varss = $variants4{"${word}_${start_time}_${et}"};
                next if (defined($processed{"${varss}_${from}_${to}"}));
                $processed{"${varss}_${from}_${to}"} = 1;
                $varss =~ s/\s+$//o;
                @vars = split(/ /, $varss);
                $hold_from = $from;
                $hold_to = $to;
                foreach $v (@vars) {
                    @parts = split(/~SIL/, $v);
                    for ($i=1; $i<=$#parts; $i++) {
                        $parts[$i] = "~SIL" . $parts[$i];  # put the SIL back
                    }
                    $from = $hold_from;
                    $final = $hold_to;
                    foreach $pa (@parts) {
                       my ($w, $s, $e) = $pa =~ /(\S+)_(\d+)_(\d+)/o; 
                       $to = ($e!=$et)?($snum++):($final); 
                       push(@output,"$s $e $w $from $to\n");
                       $from = $to;
                    }
                }
            }
        }
        @output = reverse sort link_cmp @output;
        my %seen = ();
        foreach $line (@output) {
            print $line unless ($seen{$line});
            $seen{$line} = 1;  # algorithm can produce duplicates
        }
        print ".\n";
    } else {
        print "$guid.nc NO_MATCH\n";
        print ".\n";
    }
}



