// Geoffrey Zweig (gzweig@microsoft.com)
// Copyright (c) 2009 Microsoft Corporation. All rights reserved.

#ifndef LM_H
#define LM_H

#include <vector>
#include <list>
#include <map>
#include <set>
#include <string>
#include <limits.h>

using namespace std;

typedef int lmidx;

struct lmhist;

struct succ_rec {
	int wdidx;
	lmhist *succ;
	float logp;
	bool operator < (const succ_rec b) const {return wdidx<b.wdidx;}
	succ_rec(int widx, lmhist *ngr, float c) {wdidx=widx; succ=ngr; logp=c;}
	succ_rec() {wdidx=0; succ=NULL;} // impossible word - words are negative
	succ_rec(int w) {wdidx=w; succ=NULL;}
};

struct lmhist
{
	vector<succ_rec> succs;
	succ_rec backoff;
	int lastword, idx;
	lmhist () {lastword = INT_MAX;} // impossible word - words are negative
};

struct language_model
{
    list<lmhist> ngrams;
	lmidx initial_state; // that resulting from seeing <s> initially
	set<lmidx> final_states; // those resulting from seeing </s>
	map<int, string> *word4int;  // global mapping from strings to ints
	set<int> lm_words;  // just the entities that are in the LM
	int max_ngramlen;
	vector<lmhist *> loc4;  // map from index to a structure
	vector<lmidx> path; // of last lm lookup

	lmidx lm_next_hist(lmidx, int l);	
	float lm_logp(lmidx, int l);
	int unk_idx, bos_idx, eos_idx;
	const string &unk_wrd();
        int sil_idx;

	language_model(const char *lmfile, map<string, int> &idx4word, map<int, string> &word4idx);
};


#endif


