/*
 * Patrick Nguyen (panguyen@microsoft.com)
 * Copyright (c) 2005, 2006, 2009 Microsoft Corporation.  All rights reserved.
 *
 * Decoding app
 * (it is now a swiss-army knife, switch blades using --task)
 */

#include "ngutil.h"
#include "scarf_config.h"
#include "compiler_specific.h"
#include "mpiwrap.h"

// for _setmode
#include <fcntl.h>
#ifndef _GCC_
# include <io.h>
#endif

#include <iostream>

inline void initGlobal(clparser &cl, const CFG::GlobalConfig &cfg) {

	if( !cfg.bufstdio ) {
		setvbuf(stdout, NULL, _IONBF, 2);
		setvbuf(stderr, NULL, _IONBF, 2);
		std::cout.rdbuf()->pubsetbuf(0,0);
		std::cerr.rdbuf()->pubsetbuf(0,0);
	}
	if( cfg.binstdio ) {
#ifndef _GCC_ // strictly, this will break cygwin builds
		int result;

		/* Set "stdin" to have binary mode: */
		result = setmode( fileno( stdin ), _O_BINARY );
		ERR_check(ERR::ERR_ALWAYS, result != -1, "setting mode for stdin failed");
		result = setmode( fileno( stdout ), _O_BINARY );
		ERR_check(ERR::ERR_ALWAYS, result != -1, "setting mode for stdout failed");
		result = setmode( fileno( stderr ), _O_BINARY );
		ERR_check(ERR::ERR_ALWAYS, result != -1, "setting mode for stderr failed");
#endif // _GCC_
	}

#ifndef _GCC_
	if( cfg.crtdbgbrk ) {
		// enter debugger in Debug mode only.
		_CrtDbgBreak();
	}
#endif

#ifdef __USE_MPI
	MPI::RedirectOut(cfg.mpi.redirect_log.c_str());
	MPI::InitHeartBeat(&cfg.mpi);
	if( cfg.mpi.enable ) {
		MPI::ProcessMarkups(&cfg.mpi, cl);
		fflush(stdout);fflush(stderr);
	}
#endif // __USE_MPI

}


