/*
 * Patrick Nguyen (panguyen@microsoft.com)
 * Copyright (c) 2009, 2010 Microsoft Corporation.  All rights reserved.
 */

#ifndef __MPIWRAP_H
#define __MPIWRAP_H 1

#ifndef _GCC_
# include <winsock.h>
#endif

#include "scarf_config.h"
#include "commandline.h"

class HeartBeat;

struct MPI {
	static int GetRank();
	static size_t numProcesses();
	static void Initialize(int *argc, char ***argv);
	static void Uninitialize();

	static void reduce(char &);
	static void reduce(float &);
	static void reduce(uint32 &);
	static void reduce(double &);

	static void reduce(char *, const size_t n);
	static void reduce(float *, const size_t n);
	static void reduce(uint32 *, const size_t n);
	static void reduce(double *, const size_t);
	static void bcast(char &);
	static void bcast(float &);
	static void bcast(uint32 &);
	static void bcast(double &);

	static void bcast(char *data, const size_t n);
	static void bcast(float *data, const size_t n);
	static void bcast(uint32 *data, const size_t n);
	static void bcast(double *data, const size_t n);
#if __USE_64BIT
	static void reduce(size_t &);
	static void reduce(size_t *, const size_t);
	static void bcast(size_t &);
	static void bcast(size_t *data, const size_t n);
#endif

	static void send_s(const char *, int);
	static void recv_s(char *, size_t maxlen);

	static void send(const std::vector<char> &, int);
	static void recv(std::vector<char> &);

	static void send(const size_t msg, int to);
	static void recv(size_t &msg);

	static void send(const std::vector<std::string> &list, int to);
	static void recv(std::vector<std::string> &list);

	static void childSync(int);

	static const std::string &hostname();

	static bool isRoot();
	static bool isNodeMaster();

	static void RedirectOut(const char *);

	static void MakeLocalDir(const CFG::MPIConfig *mpicfg);
	static void LocalizePath(char *dest, const size_t maxdestlen, const char *localDir, const char *origFileName);
	static void GetLocalDir(const CFG::MPIConfig *mpicfg, char *dest, const size_t destlen);
	static void CopyFileLocally(const CFG::MPIConfig *mpicfg, const char *fn);
	static void CopyFromCache(const CFG::MPIConfig *mpicfg, const char *fn); // copy from node cache to new location

	static void SyncPoint();

	static void BCastFile(const CFG::MPIConfig *mpicfg, const char *fn);
	static std::string GetLocalFileName(const CFG::MPIConfig *mpicfg, const char *fn);

	// dispatch, cache locally, or broadast upon request
	static void ProcessMarkups(const CFG::MPIConfig *mpicfg, clparser &cl);

	static void InitHeartBeat(const CFG::MPIConfig *mpicfg);
	static void UnInitHeartBeat();

	static const int ROOT_RANK = 0;
protected:
	static int rankId_;
	static int numProcess_;

	static std::string hostname_;


	static bool isNodeMaster_; // a node master is in charge of common resources in a node

	static HeartBeat *heartBeat_;
};

#endif // __MPIWRAP_H
