/*
 * Patrick Nguyen (panguyen@microsoft.com)
 * Copyright (c) 2005 Microsoft Corporation.  All rights reserved.
 *
 * Next Generation Decoder Basic types
 */

#ifndef __NGTYPES_H
#define __NGTYPES_H 1

#ifndef _GCC_
// testing constants
# pragma warning( disable: 4127 )
// assignment operator could not be generated
# pragma warning( disable: 4512 )
// decorated names
# pragma warning( disable: 4503 )
// rethrow error
# pragma warning( disable: 4571 )
// expression has no effect; expected expression with side-effect
# pragma warning( disable: 4555 )
// not inlined, and inlined automatically
# pragma warning( disable: 4710 4711 )
//alignment -- happens in system headers
//#pragma warning(disable: 4820)

// PREfast (analyze) might not be enabled
//#pragma warning( disable: 4609 )
// leaking due to an exception ! ouch!
# pragma warning( disable: 6211 )
// potential comparison of a constant with another constant and other trivial tests
# pragma warning( disable: 6326 6329 6237 6239 )
// in system headers, NULL dereferencing
# pragma warning( disable: 6011 )
// stack too large (mostly due to MAX_LINE_LENGTH)
# pragma warning( disable: 6262 )

// unreferenced inline function has been removed
# pragma warning (disable: 4514)

// to compile in dbg with /Wall
// comma in std::vector
# pragma warning( disable: 4548 )

// padding added after data member
# pragma warning( disable: 4820 )

//conversion signed/unsigned mismatch
# pragma warning( disable: 4365 )

//assignment operator could not be generated because a base class assignment operator is inaccessible
# pragma warning( disable: 4625 4626 )
//unreferenced local function has been removed
# pragma warning( disable: 4505 )
//enumerator is not explicitly handled by a case label (/analyze)
# pragma warning( disable: 4061 )
//there is no warning number 4609?
# pragma warning( disable: 4619 )

# pragma warning(push)
#  pragma warning(disable: 4820) // 4609
#  include <wchar.h>
# pragma warning(pop)

// there's no way to disable this locally, this is about using unsafe std constructs that
// write on unsafe arrays
# pragma warning(disable: 4996)

// pngT: ouch! when was this fixed? size_t -> uint
# pragma warning(disable: 4267 )

#endif // warning tweak

#include <assert.h>

#ifndef _GCC_
# pragma warning(push)
# pragma warning(disable: 4820) // 4609
#endif
#include <string>
#ifndef _GCC_
#pragma warning(pop)
#endif 

#include <map>
#include <vector>
#include <deque>
#include <queue>
#include <algorithm>
#include <set>
#include <stack>
#include <new>
#include <numeric>
#include <functional>
#include <cmath>
#include <cstdlib>
#include <float.h> // for _finite(), _isnan()
#include <limits.h> 
#include <string.h>
#include <stdio.h>

typedef unsigned char       uchar;
typedef unsigned short int ushort;
typedef unsigned int         uint;
typedef signed int          int32;
typedef unsigned int       uint32;
typedef unsigned short int uint16;
typedef signed short int    int16;
typedef unsigned char       uint8;
typedef signed char          int8;

typedef float              real32;
typedef double             real64;

typedef signed long long   int64;
typedef unsigned long long uint64;

namespace NGTypes {

    const int MAX_LINE_LENGTH = 16384;

} // NGTypes namespace

using namespace NGTypes;

// appendum: error handling
#include "ngerr.h"

#endif /* __NGTYPES_H */
